Class: BigDecimal
- Inherits:
-
Numeric
- Object
- Numeric
- BigDecimal
- Defined in:
- ext/bigdecimal/bigdecimal.c,
lib/bigdecimal.rb,
lib/bigdecimal.rb,
lib/bigdecimal/util.rb,
sig/big_decimal.rbs,
sig/big_decimal_util.rbs,
ext/bigdecimal/bigdecimal.c
Overview
BigDecimal provides arbitrary-precision floating point decimal arithmetic.
Introduction
Ruby provides built-in support for arbitrary precision integer arithmetic.
For example:
42**13 #=> 1265437718438866624512
BigDecimal provides similar support for very large or very accurate floating point numbers.
Decimal arithmetic is also useful for general calculation, because it provides the correct answers people expect--whereas normal binary floating point arithmetic often introduces subtle errors because of the conversion between base 10 and base 2.
For example, try:
sum = 0
10_000.times do
sum = sum + 0.0001
end
print sum #=> 0.9999999999999062
and contrast with the output from:
require 'bigdecimal'
sum = BigDecimal("0")
10_000.times do
sum = sum + BigDecimal("0.0001")
end
print sum #=> 0.1E1
Similarly:
(BigDecimal("1.2") - BigDecimal("1.0")) == BigDecimal("0.2") #=> true
(1.2 - 1.0) == 0.2 #=> false
A Note About Precision
For a calculation using a BigDecimal and another value,
the precision of the result depends on the type of value:
- If
valueis a Float, the precision is Float::DIG + 1. - If
valueis a Rational, the precision is larger than Float::DIG + 1. - If
valueis a BigDecimal, the precision is +value+'s precision in the internal representation, which is platform-dependent. - If
valueis other object, the precision is determined by the result ofBigDecimal(value).
Special features of accurate decimal arithmetic
Because BigDecimal is more accurate than normal binary floating point arithmetic, it requires some special values.
Infinity
BigDecimal sometimes needs to return infinity, for example if you divide a value by zero.
BigDecimal("1.0") / BigDecimal("0.0") #=> Infinity BigDecimal("-1.0") / BigDecimal("0.0") #=> -Infinity
You can represent infinite numbers to BigDecimal using the strings
'Infinity', '+Infinity' and
'-Infinity' (case-sensitive)
Not a Number
When a computation results in an undefined value, the special value NaN
(for 'not a number') is returned.
Example:
BigDecimal("0.0") / BigDecimal("0.0") #=> NaN
You can also create undefined values.
NaN is never considered to be the same as any other value, even NaN itself:
n = BigDecimal('NaN') n == 0.0 #=> false n == n #=> false
Positive and negative zero
If a computation results in a value which is too small to be represented as a BigDecimal within the currently specified limits of precision, zero must be returned.
If the value which is too small to be represented is negative, a BigDecimal value of negative zero is returned.
BigDecimal("1.0") / BigDecimal("-Infinity") #=> -0.0
If the value is positive, a value of positive zero is returned.
BigDecimal("1.0") / BigDecimal("Infinity") #=> 0.0
(See BigDecimal.mode for how to specify limits of precision.)
Note that -0.0 and 0.0 are considered to be the same for the purposes of
comparison.
Note also that in mathematics, there is no particular concept of negative or positive zero; true mathematical zero has no sign.
bigdecimal/util
When you require bigdecimal/util, the #to_d method will be
available on BigDecimal and the native Integer, Float, Rational,
String, Complex, and NilClass classes:
require 'bigdecimal/util'
42.to_d # => 0.42e2
0.5.to_d # => 0.5e0
(2/3r).to_d(3) # => 0.667e0
"0.5".to_d # => 0.5e0
Complex(0.1234567, 0).to_d(4) # => 0.1235e0
nil.to_d # => 0.0
Methods for Working with JSON
- ::json_create: Returns a new BigDecimal object constructed from the given object.
- #as_json:
Returns a 2-element hash representing
self. - #to_json:
Returns a JSON string representing
self.
These methods are provided by the JSON gem. To make these methods available:
require 'json/add/bigdecimal'
-
License
Copyright (C) 2002 by Shigeo Kobayashi [email protected].
BigDecimal is released under the Ruby and 2-clause BSD licenses. See LICENSE.txt for details.
Maintained by mrkn [email protected] and ruby-core members.
Documented by zzak [email protected], mathew [email protected], and many other contributors.
Defined Under Namespace
Modules: Internal
Constant Summary collapse
- BASE =
Base value used in internal calculations. On a 32 bit system, BASE is 10000, indicating that calculation is done in groups of 4 digits. (If it were larger, BASE**2 wouldn't fit in 32 bits, so you couldn't guarantee that two groups could always be multiplied together without overflow.)
INT2FIX((SIGNED_VALUE)BASE)
- EXCEPTION_ALL =
Determines whether overflow, underflow or zero divide result in an exception being thrown. See BigDecimal.mode.
0xff- EXCEPTION_INFINITY =
Determines what happens when the result of a computation is infinity. See BigDecimal.mode.
0x01- EXCEPTION_NaN =
Determines what happens when the result of a computation is not a number (NaN). See BigDecimal.mode.
0x02- EXCEPTION_OVERFLOW =
Determines what happens when the result of a computation is an overflow (a result too large to be represented). See BigDecimal.mode.
0x01- EXCEPTION_UNDERFLOW =
Determines what happens when the result of a computation is an underflow (a result too small to be represented). See BigDecimal.mode.
0x04- EXCEPTION_ZERODIVIDE =
Determines what happens when a division by zero is performed. See BigDecimal.mode.
0x10- INFINITY =
BigDecimal@Infinity] value.
Positive infinity[rdoc-ref
- NAN =
BigDecimal@Not+a+Number]' value.
'{Not a Number}[rdoc-ref- ROUND_CEILING =
Round towards +Infinity. See BigDecimal.mode.
5- ROUND_DOWN =
Indicates that values should be rounded towards zero. See BigDecimal.mode.
2- ROUND_FLOOR =
Round towards -Infinity. See BigDecimal.mode.
6- ROUND_HALF_DOWN =
Indicates that digits >= 6 should be rounded up, others rounded down. See BigDecimal.mode.
4- ROUND_HALF_EVEN =
Round towards the even neighbor. See BigDecimal.mode.
7- ROUND_HALF_UP =
Indicates that digits >= 5 should be rounded up, others rounded down. See BigDecimal.mode.
3- ROUND_MODE =
Determines what happens when a result must be rounded in order to fit in the appropriate number of significant digits. See BigDecimal.mode.
0x100- ROUND_UP =
Indicates that values should be rounded away from zero. See BigDecimal.mode.
1- SIGN_NEGATIVE_FINITE =
Indicates that a value is negative and finite. See BigDecimal.sign.
-2
- SIGN_NEGATIVE_INFINITE =
Indicates that a value is negative and infinite. See BigDecimal.sign.
-3
- SIGN_NEGATIVE_ZERO =
Indicates that a value is -0. See BigDecimal.sign.
-1
- SIGN_NaN =
Indicates that a value is not a number. See BigDecimal.sign.
0- SIGN_POSITIVE_FINITE =
Indicates that a value is positive and finite. See BigDecimal.sign.
2- SIGN_POSITIVE_INFINITE =
Indicates that a value is positive and infinite. See BigDecimal.sign.
3- SIGN_POSITIVE_ZERO =
Indicates that a value is +0. See BigDecimal.sign.
1- VERSION =
The version of bigdecimal library
rb_str_new2(BIGDECIMAL_VERSION)
Class Method Summary collapse
-
._load(str) ⇒ Object
Internal method used to provide marshalling support.
-
.double_fig ⇒ Integer
Returns the number of digits a Float object is allowed to have; the result is system-dependent:.
-
.interpret_loosely(string) ⇒ Object
Returns the
BigDecimalconverted loosely fromstring. -
.limit(digits) ⇒ Object
Limit the number of significant digits in newly created BigDecimal numbers to the specified value.
-
.mode(mode, setting = nil) ⇒ Integer
Returns an integer representing the mode settings for exception handling and rounding.
-
.save_exception_mode { ... } ⇒ Object
Execute the provided block, but preserve the exception mode.
-
.save_limit { ... } ⇒ Object
Execute the provided block, but preserve the precision limit.
-
.save_rounding_mode { ... } ⇒ Object
Execute the provided block, but preserve the rounding mode.
Instance Method Summary collapse
-
#% ⇒ Object
%: a%b = a - (a.to_f/b).floor * b.
-
#*(b) ⇒ Object
Multiply by the specified value.
-
#**(y) ⇒ Object
Returns the BigDecimal value of `self` raised to power `other`:.
-
#+(value) ⇒ Object
Returns the BigDecimal sum of
selfandvalue:. -
#+ ⇒ self
Returns
self:. -
#-(value) ⇒ Object
Returns the BigDecimal difference of
selfandvalue:. -
#- ⇒ Object
Returns the BigDecimal negation of self:.
-
#/ ⇒ Object
For c = self/r: with round operation.
-
#<(other) ⇒ Boolean
Returns
trueifselfis less thanother,falseotherwise:. -
#<=(other) ⇒ Boolean
Returns
trueifselfis less or equal to thanother,falseotherwise:. -
#<=>(r) ⇒ Object
The comparison operator.
-
#==(r) ⇒ Object
Tests for value equality; returns true if the values are equal.
-
#===(r) ⇒ Object
Tests for value equality; returns true if the values are equal.
-
#>(other) ⇒ Boolean
Returns
trueifselfis greater thanother,falseotherwise:. -
#>=(other) ⇒ Boolean
Returns
trueifselfis greater than or equal toother,falseotherwise:. -
#_decimal_shift(v) ⇒ Object
Returns self * 10**v without changing the precision.
-
#_dump ⇒ String
Returns a string representing the marshalling of
self. -
#abs ⇒ Object
Returns the BigDecimal absolute value of
self:. -
#add(value, ndigits) ⇒ Object
Returns the BigDecimal sum of
selfandvaluewith a precision ofndigitsdecimal digits. -
#ceil(n) ⇒ Object
Return the smallest integer greater than or equal to the value, as a BigDecimal.
-
#clone ⇒ Object
:nodoc:.
-
#coerce(other) ⇒ Object
The coerce method provides support for Ruby type coercion.
-
#div(*args) ⇒ Object
call-seq: div(value) -> integer div(value, digits) -> bigdecimal or integer.
-
#divmod(value) ⇒ Object
Divides by the specified value, and returns the quotient and modulus as BigDecimal numbers.
-
#dup ⇒ Object
:nodoc:.
-
#eql?(r) ⇒ Boolean
Tests for value equality; returns true if the values are equal.
-
#exponent ⇒ Object
Returns the exponent of the BigDecimal number, as an Integer.
-
#finite? ⇒ Boolean
Returns True if the value is finite (not NaN or infinite).
-
#fix ⇒ Object
Return the integer part of the number, as a BigDecimal.
-
#floor(n) ⇒ Object
Return the largest integer less than or equal to the value, as a BigDecimal.
-
#frac ⇒ Object
Return the fractional part of the number, as a BigDecimal.
-
#hash ⇒ Integer
Returns the integer hash value for
self. -
#infinite? ⇒ Boolean
Returns nil, -1, or +1 depending on whether the value is finite, -Infinity, or +Infinity.
- #initialize_copy ⇒ self
-
#inspect ⇒ Object
Returns a string representation of self.
-
#modulo ⇒ Object
%: a%b = a - (a.to_f/b).floor * b.
-
#mult(other, ndigits) ⇒ Object
Returns the BigDecimal product of
selfandvaluewith a precision ofndigitsdecimal digits. -
#n_significant_digits ⇒ Integer
Returns the number of decimal significant digits in
self. -
#nan? ⇒ Boolean
Returns True if the value is Not a Number.
- #newton_raphson_inverse(prec) ⇒ Object
-
#nonzero? ⇒ Boolean
Returns self if the value is non-zero, nil otherwise.
- #nttmult(v) ⇒ Object
-
#power(y, prec = 0) ⇒ BigDecimal
Returns the value raised to the power of n.
-
#precision ⇒ Integer
Returns the number of decimal digits in
self:. -
#precision_scale ⇒ Array
Returns a 2-length array; the first item is the result of BigDecimal#precision and the second one is of BigDecimal#scale.
-
#quo(*args) ⇒ Object
Divide by the specified value.
-
#remainder ⇒ Object
remainder.
-
#round(n, mode) ⇒ Object
Round to the nearest integer (by default), returning the result as a BigDecimal if n is specified and positive, or as an Integer if it isn't.
-
#scale ⇒ Integer
Returns the number of decimal digits following the decimal digits in
self. -
#sign ⇒ Object
Returns the sign of the value.
-
#split ⇒ Object
Splits a BigDecimal number into four parts, returned as an array of values.
-
#sqrt(prec) ⇒ BigDecimal
Returns the square root of the value.
-
#sub(value, digits) ⇒ Object
Subtract the specified value.
-
#to_d ⇒ BigDecimal
Returns self.
-
#to_digits ⇒ String
Converts a BigDecimal to a String of the form "nnnnnn.mmm".
-
#to_f ⇒ Object
Returns a new Float object having approximately the same value as the BigDecimal number.
-
#to_i ⇒ Object
Returns the value as an Integer.
-
#to_int ⇒ Object
Returns the value as an Integer.
-
#to_r ⇒ Object
Converts a BigDecimal to a Rational.
-
#to_s(s) ⇒ Object
Converts the value to a string.
-
#truncate(n) ⇒ Object
Truncate to the nearest integer (by default), returning the result as a BigDecimal.
- #vpdivd(r, cprec) ⇒ Object
- #vpdivd_newton(r, cprec) ⇒ Object
- #vpmult(v) ⇒ Object
-
#zero? ⇒ Boolean
Returns True if the value is zero.
Class Method Details
._load(str) ⇒ Object
Internal method used to provide marshalling support. See the Marshal module.
13 |
# File 'sig/big_decimal.rbs', line 13
def self._load: (String) -> BigDecimal
|
.double_fig ⇒ Integer
Returns the number of digits a Float object is allowed to have; the result is system-dependent:
BigDecimal.double_fig # => 16
24 |
# File 'sig/big_decimal.rbs', line 24
def self.double_fig: () -> Integer
|
.interpret_loosely(string) ⇒ Object
Returns the BigDecimal converted loosely from string.
31 |
# File 'sig/big_decimal.rbs', line 31
def self.interpret_loosely: (string) -> BigDecimal
|
.limit(digits) ⇒ Object
Limit the number of significant digits in newly created BigDecimal numbers to the specified value. Rounding is performed as necessary, as specified by BigDecimal.mode.
A limit of 0, the default, means no upper limit.
The limit specified by this method takes less priority over any limit specified to instance methods such as ceil, floor, truncate, or round.
46 |
# File 'sig/big_decimal.rbs', line 46
def self.limit: (?Integer? digits) -> Integer
|
.mode(mode, setting = nil) ⇒ Integer
Returns an integer representing the mode settings for exception handling and rounding.
These modes control exception handling:
- BigDecimal::EXCEPTION_NaN.
- BigDecimal::EXCEPTION_INFINITY.
- BigDecimal::EXCEPTION_UNDERFLOW.
- BigDecimal::EXCEPTION_OVERFLOW.
- BigDecimal::EXCEPTION_ZERODIVIDE.
- BigDecimal::EXCEPTION_ALL.
Values for setting for exception handling:
true: sets the givenmodetotrue.false: sets the givenmodetofalse.nil: does not modify the mode settings.
You can use method BigDecimal.save_exception_mode to temporarily change, and then automatically restore, exception modes.
For clarity, some examples below begin by setting all
exception modes to false.
This mode controls the way rounding is to be performed:
- BigDecimal::ROUND_MODE
You can use method BigDecimal.save_rounding_mode to temporarily change, and then automatically restore, the rounding mode.
NaNs
Mode BigDecimal::EXCEPTION_NaN controls behavior when a BigDecimal NaN is created.
Settings:
false(default): Returns BigDecimal('NaN').true: Raises FloatDomainError.
Examples:
BigDecimal.mode(BigDecimal::EXCEPTION_ALL, false) # => 0
BigDecimal('NaN') # => NaN
BigDecimal.mode(BigDecimal::EXCEPTION_NaN, true) # => 2
BigDecimal('NaN') # Raises FloatDomainError
Infinities
Mode BigDecimal::EXCEPTION_INFINITY controls behavior when a BigDecimal Infinity or -Infinity is created. Settings:
false(default): Returns BigDecimal('Infinity') or BigDecimal('-Infinity').true: Raises FloatDomainError.
Examples:
BigDecimal.mode(BigDecimal::EXCEPTION_ALL, false) # => 0
BigDecimal('Infinity') # => Infinity
BigDecimal('-Infinity') # => -Infinity
BigDecimal.mode(BigDecimal::EXCEPTION_INFINITY, true) # => 1
BigDecimal('Infinity') # Raises FloatDomainError
BigDecimal('-Infinity') # Raises FloatDomainError
Underflow
Mode BigDecimal::EXCEPTION_UNDERFLOW controls behavior when a BigDecimal underflow occurs. Settings:
false(default): Returns BigDecimal('0') or BigDecimal('-Infinity').true: Raises FloatDomainError.
Examples:
BigDecimal.mode(BigDecimal::EXCEPTION_ALL, false) # => 0
def flow_under
x = BigDecimal('0.1')
100.times { x *= x }
end
flow_under # => 100
BigDecimal.mode(BigDecimal::EXCEPTION_UNDERFLOW, true) # => 4
flow_under # Raises FloatDomainError
Overflow
Mode BigDecimal::EXCEPTION_OVERFLOW controls behavior when a BigDecimal overflow occurs. Settings:
false(default): Returns BigDecimal('Infinity') or BigDecimal('-Infinity').true: Raises FloatDomainError.
Examples:
BigDecimal.mode(BigDecimal::EXCEPTION_ALL, false) # => 0
def flow_over
x = BigDecimal('10')
100.times { x *= x }
end
flow_over # => 100
BigDecimal.mode(BigDecimal::EXCEPTION_OVERFLOW, true) # => 1
flow_over # Raises FloatDomainError
Zero Division
Mode BigDecimal::EXCEPTION_ZERODIVIDE controls behavior when a zero-division occurs. Settings:
false(default): Returns BigDecimal('Infinity') or BigDecimal('-Infinity').true: Raises FloatDomainError.
Examples:
BigDecimal.mode(BigDecimal::EXCEPTION_ALL, false) # => 0
one = BigDecimal('1')
zero = BigDecimal('0')
one / zero # => Infinity
BigDecimal.mode(BigDecimal::EXCEPTION_ZERODIVIDE, true) # => 16
one / zero # Raises FloatDomainError
All Exceptions
Mode BigDecimal::EXCEPTION_ALL controls all of the above:
BigDecimal.mode(BigDecimal::EXCEPTION_ALL, false) # => 0
BigDecimal.mode(BigDecimal::EXCEPTION_ALL, true) # => 23
Rounding
Mode BigDecimal::ROUND_MODE controls the way rounding is to be performed;
its setting values are:
ROUND_UP: Round away from zero. Aliased as:up.ROUND_DOWN: Round toward zero. Aliased as:downand:truncate.ROUND_HALF_UP: Round toward the nearest neighbor; if the neighbors are equidistant, round away from zero. Aliased as:half_upand:default.ROUND_HALF_DOWN: Round toward the nearest neighbor; if the neighbors are equidistant, round toward zero. Aliased as:half_down.ROUND_HALF_EVEN(Banker's rounding): Round toward the nearest neighbor; if the neighbors are equidistant, round toward the even neighbor. Aliased as:half_evenand:banker.ROUND_CEILING: Round toward positive infinity. Aliased as:ceilingand:ceil.ROUND_FLOOR: Round toward negative infinity. Aliased as:floor:.
200 201 |
# File 'sig/big_decimal.rbs', line 200
def self.mode: (round_mode, ?(round_mode_integer | round_mode_symbol)) -> Integer
| (Integer exception_mode, ?bool? setting) -> Integer
|
.save_exception_mode { ... } ⇒ Object
Execute the provided block, but preserve the exception mode
BigDecimal.save_exception_mode do
BigDecimal.mode(BigDecimal::EXCEPTION_OVERFLOW, false)
BigDecimal.mode(BigDecimal::EXCEPTION_NaN, false)
BigDecimal(BigDecimal('Infinity'))
BigDecimal(BigDecimal('-Infinity'))
BigDecimal(BigDecimal('NaN'))
end
For use with the BigDecimal::EXCEPTION_*
See BigDecimal.mode
222 |
# File 'sig/big_decimal.rbs', line 222
def self.save_exception_mode: () { (?nil) -> void } -> void
|
.save_limit { ... } ⇒ Object
Execute the provided block, but preserve the precision limit
BigDecimal.limit(100)
puts BigDecimal.limit
BigDecimal.save_limit do
BigDecimal.limit(200)
puts BigDecimal.limit
end
puts BigDecimal.limit
238 |
# File 'sig/big_decimal.rbs', line 238
def self.save_limit: () { (?nil) -> void } -> void
|
.save_rounding_mode { ... } ⇒ Object
Execute the provided block, but preserve the rounding mode
BigDecimal.save_rounding_mode do
BigDecimal.mode(BigDecimal::ROUND_MODE, :up)
puts BigDecimal.mode(BigDecimal::ROUND_MODE)
end
For use with the BigDecimal::ROUND_*
See BigDecimal.mode
255 |
# File 'sig/big_decimal.rbs', line 255
def self.save_rounding_mode: () { (?nil) -> void } -> void
|
Instance Method Details
#% ⇒ Object
%: a%b = a - (a.to_f/b).floor * b
266 |
# File 'sig/big_decimal.rbs', line 266
def %: (real | BigDecimal) -> BigDecimal
|
#*(b) ⇒ Object
Multiply by the specified value.
The result precision will be the precision of the sum of each precision.
See BigDecimal#mult.
273 274 |
# File 'sig/big_decimal.rbs', line 273
def *: (real | BigDecimal) -> BigDecimal
| (Complex) -> Complex
|
#**(arg0) ⇒ BigDecimal #**(arg0) ⇒ Complex
Returns the BigDecimal value of self raised to power other:
b = BigDecimal('3.14')
b ** 2 # => 0.98596e1
b ** 2.0 # => 0.98596e1
b ** Rational(2, 1) # => 0.98596e1
Related: BigDecimal#power.
127 128 129 130 131 132 133 134 135 136 137 |
# File 'lib/bigdecimal.rb', line 127 def **(y) case y when BigDecimal, Integer, Float, Rational power(y) when nil raise TypeError, 'wrong argument type NilClass' else x, y = y.coerce(self) x**y end end |
#+(value) ⇒ Object
Returns the BigDecimal sum of self and value:
b = BigDecimal('111111.111') # => 0.111111111e6
b + 2 # => 0.111113111e6
b + 2.0 # => 0.111113111e6
b + Rational(2, 1) # => 0.111113111e6
b + Complex(2, 0) # => (0.111113111e6+0i)
See the Note About Precision.
307 308 |
# File 'sig/big_decimal.rbs', line 307
def +: (real | BigDecimal) -> BigDecimal
| (Complex) -> Complex
|
#+ ⇒ self
Returns self:
+BigDecimal(5) # => 0.5e1
+BigDecimal(-5) # => -0.5e1
319 |
# File 'sig/big_decimal.rbs', line 319
def +@: () -> BigDecimal
|
#-(value) ⇒ Object
Returns the BigDecimal difference of self and value:
b = BigDecimal('333333.333') # => 0.333333333e6
b - 2 # => 0.333331333e6
b - 2.0 # => 0.333331333e6
b - Rational(2, 1) # => 0.333331333e6
b - Complex(2, 0) # => (0.333331333e6+0i)
See the Note About Precision.
336 337 |
# File 'sig/big_decimal.rbs', line 336
def -: (real | BigDecimal) -> BigDecimal
| (Complex) -> Complex
|
#- ⇒ Object
Returns the BigDecimal negation of self:
b0 = BigDecimal('1.5')
b1 = -b0 # => -0.15e1
b2 = -b1 # => 0.15e1
349 |
# File 'sig/big_decimal.rbs', line 349
def -@: () -> BigDecimal
|
#/ ⇒ Object
For c = self/r: with round operation
362 363 |
# File 'sig/big_decimal.rbs', line 362
def /: (real | BigDecimal) -> BigDecimal
| (Complex) -> Complex
|
#<(other) ⇒ Boolean
Returns true if self is less than other, false otherwise:
b = BigDecimal('1.5') # => 0.15e1
b < 2 # => true
b < 2.0 # => true
b < Rational(2, 1) # => true
b < 1.5 # => false
Raises an exception if the comparison cannot be made.
379 |
# File 'sig/big_decimal.rbs', line 379
def <: (real | BigDecimal) -> bool
|
#<=(other) ⇒ Boolean
Returns true if self is less or equal to than other, false otherwise:
b = BigDecimal('1.5') # => 0.15e1
b <= 2 # => true
b <= 2.0 # => true
b <= Rational(2, 1) # => true
b <= 1.5 # => true
b < 1 # => false
Raises an exception if the comparison cannot be made.
396 |
# File 'sig/big_decimal.rbs', line 396
def <=: (real | BigDecimal) -> bool
|
#<=>(r) ⇒ Object
The comparison operator. a <=> b is 0 if a == b, 1 if a > b, -1 if a < b.
404 |
# File 'sig/big_decimal.rbs', line 404
def <=>: (untyped) -> Integer?
|
#==(r) ⇒ Object
Tests for value equality; returns true if the values are equal.
The == and === operators and the eql? method have the same implementation for BigDecimal.
Values may be coerced to perform the comparison:
BigDecimal('1.0') == 1.0 #=> true
419 |
# File 'sig/big_decimal.rbs', line 419
def ==: (untyped) -> bool
|
#===(r) ⇒ Object
Tests for value equality; returns true if the values are equal.
The == and === operators and the eql? method have the same implementation for BigDecimal.
Values may be coerced to perform the comparison:
BigDecimal('1.0') == 1.0 #=> true
431 |
# File 'sig/big_decimal.rbs', line 431
def ===: (untyped) -> bool
|
#>(other) ⇒ Boolean
Returns true if self is greater than other, false otherwise:
b = BigDecimal('1.5')
b > 1 # => true
b > 1.0 # => true
b > Rational(1, 1) # => true
b > 2 # => false
Raises an exception if the comparison cannot be made.
447 |
# File 'sig/big_decimal.rbs', line 447
def >: (real | BigDecimal) -> bool
|
#>=(other) ⇒ Boolean
Returns true if self is greater than or equal to other, false otherwise:
b = BigDecimal('1.5')
b >= 1 # => true
b >= 1.0 # => true
b >= Rational(1, 1) # => true
b >= 1.5 # => true
b > 2 # => false
Raises an exception if the comparison cannot be made.
465 |
# File 'sig/big_decimal.rbs', line 465
def >=: (real | BigDecimal) -> bool
|
#_decimal_shift(v) ⇒ Object
Returns self * 10**v without changing the precision. This method is currently for internal use.
BigDecimal("0.123e10")._decimal_shift(20) #=> "0.123e30"
BigDecimal("0.123e10")._decimal_shift(-20) #=> "0.123e-10"
5 6 7 |
# File 'lib/bigdecimal.rb', line 5 def _decimal_shift(i) # :nodoc: to_java.move_point_right(i).to_d end |
#_dump ⇒ String
Returns a string representing the marshalling of self.
See module Marshal.
inf = BigDecimal('Infinity') # => Infinity
dumped = inf._dump # => "9:Infinity"
BigDecimal._load(dumped) # => Infinity
477 |
# File 'sig/big_decimal.rbs', line 477
def _dump: (?untyped) -> String
|
#abs ⇒ Object
Returns the BigDecimal absolute value of self:
BigDecimal('5').abs # => 0.5e1
BigDecimal('-3').abs # => 0.3e1
488 |
# File 'sig/big_decimal.rbs', line 488
def abs: () -> BigDecimal
|
#add(value, ndigits) ⇒ Object
Returns the BigDecimal sum of self and value
with a precision of ndigits decimal digits.
When ndigits is less than the number of significant digits
in the sum, the sum is rounded to that number of digits,
according to the current rounding mode; see BigDecimal.mode.
Examples:
# Set the rounding mode.
BigDecimal.mode(BigDecimal::ROUND_MODE, :half_up)
b = BigDecimal('111111.111')
b.add(1, 0) # => 0.111112111e6
b.add(1, 3) # => 0.111e6
b.add(1, 6) # => 0.111112e6
b.add(1, 15) # => 0.111112111e6
b.add(1.0, 15) # => 0.111112111e6
b.add(Rational(1, 1), 15) # => 0.111112111e6
513 |
# File 'sig/big_decimal.rbs', line 513
def add: (real | BigDecimal value, Integer digits) -> BigDecimal
|
#ceil(n) ⇒ Object
Return the smallest integer greater than or equal to the value, as a BigDecimal.
BigDecimal('3.14159').ceil #=> 4 BigDecimal('-9.1').ceil #=> -9
If n is specified and positive, the fractional part of the result has no more than that many digits.
If n is specified and negative, at least that many digits to the left of the decimal point will be 0 in the result.
BigDecimal('3.14159').ceil(3) #=> 3.142 BigDecimal('13345.234').ceil(-2) #=> 13400.0
534 535 |
# File 'sig/big_decimal.rbs', line 534
def ceil: () -> Integer
| (int n) -> BigDecimal
|
#clone ⇒ Object
:nodoc:
542 |
# File 'sig/big_decimal.rbs', line 542
def clone: () -> self
|
#coerce(other) ⇒ Object
The coerce method provides support for Ruby type coercion. It is not enabled by default.
This means that binary operations like + * / or - can often be performed on a BigDecimal and an object of another type, if the other object can be coerced into a BigDecimal value.
e.g. a = BigDecimal("1.0") b = a / 2.0 #=> 0.5
Note that coercing a String to a BigDecimal is not supported by default; it requires a special compile-time option when building Ruby.
562 |
# File 'sig/big_decimal.rbs', line 562
def coerce: (Numeric) -> [ BigDecimal, BigDecimal ]
|
#div(*args) ⇒ Object
call-seq:
div(value) -> integer
div(value, digits) -> bigdecimal or integer
Divide by the specified value.
- digits
If specified and less than the number of significant digits of the result, the result is rounded to that number of digits, according to BigDecimal.mode.
If digits is 0, the result is the same as for the / operator
or #quo.
If digits is not specified, the result is an integer,
by analogy with Float#div; see also BigDecimal#divmod.
See BigDecimal#/. See BigDecimal#quo.
Examples:
a = BigDecimal("4")
b = BigDecimal("3")
a.div(b, 3) # => 0.133e1
a.div(b, 0) # => 0.1333333333333333333e1
a / b # => 0.1333333333333333333e1
a.quo(b) # => 0.1333333333333333333e1
a.div(b) # => 1
597 598 |
# File 'sig/big_decimal.rbs', line 597
def div: (real | BigDecimal value) -> Integer
| (real | BigDecimal value, int digits) -> BigDecimal
|
#divmod(value) ⇒ Object
Divides by the specified value, and returns the quotient and modulus as BigDecimal numbers. The quotient is rounded towards negative infinity.
For example:
require 'bigdecimal'
a = BigDecimal("42")
b = BigDecimal("9")
q, m = a.divmod(b)
c = q * b + m
a == c #=> true
The quotient q is (a/b).floor, and the modulus is the amount that must be added to q * b to get a.
623 |
# File 'sig/big_decimal.rbs', line 623
def divmod: (real | BigDecimal) -> [ Integer, BigDecimal ]
|
#dup ⇒ Object
:nodoc:
630 |
# File 'sig/big_decimal.rbs', line 630
def dup: () -> self
|
#eql?(r) ⇒ Boolean
Tests for value equality; returns true if the values are equal.
The == and === operators and the eql? method have the same implementation for BigDecimal.
Values may be coerced to perform the comparison:
BigDecimal('1.0') == 1.0 #=> true
642 |
# File 'sig/big_decimal.rbs', line 642
def eql?: (untyped) -> bool
|
#exponent ⇒ Object
Returns the exponent of the BigDecimal number, as an Integer.
If the number can be represented as 0.xxxxxx*10**n where xxxxxx is a string of digits with no leading zeros, then n is the exponent.
653 |
# File 'sig/big_decimal.rbs', line 653
def exponent: () -> Integer
|
#finite? ⇒ Boolean
Returns True if the value is finite (not NaN or infinite).
661 |
# File 'sig/big_decimal.rbs', line 661
def finite?: () -> bool
|
#fix ⇒ Object
Return the integer part of the number, as a BigDecimal.
669 |
# File 'sig/big_decimal.rbs', line 669
def fix: () -> BigDecimal
|
#floor(n) ⇒ Object
Return the largest integer less than or equal to the value, as a BigDecimal.
BigDecimal('3.14159').floor #=> 3 BigDecimal('-9.1').floor #=> -10
If n is specified and positive, the fractional part of the result has no more than that many digits.
If n is specified and negative, at least that many digits to the left of the decimal point will be 0 in the result.
BigDecimal('3.14159').floor(3) #=> 3.141 BigDecimal('13345.234').floor(-2) #=> 13300.0
689 690 |
# File 'sig/big_decimal.rbs', line 689
def floor: () -> Integer
| (int n) -> BigDecimal
|
#frac ⇒ Object
Return the fractional part of the number, as a BigDecimal.
698 |
# File 'sig/big_decimal.rbs', line 698
def frac: () -> BigDecimal
|
#hash ⇒ Integer
Returns the integer hash value for self.
Two instances of BigDecimal have the same hash value if and only if they have equal:
- Sign.
- Fractional part.
- Exponent.
713 |
# File 'sig/big_decimal.rbs', line 713
def hash: () -> Integer
|
#infinite? ⇒ Boolean
Returns nil, -1, or +1 depending on whether the value is finite, -Infinity, or +Infinity.
722 |
# File 'sig/big_decimal.rbs', line 722
def infinite?: () -> Integer?
|
#initialize_copy ⇒ self
1052 |
# File 'sig/big_decimal.rbs', line 1052
def initialize_copy: (self) -> self
|
#inspect ⇒ Object
Returns a string representation of self.
BigDecimal("1234.5678").inspect
#=> "0.12345678e4"
733 |
# File 'sig/big_decimal.rbs', line 733
def inspect: () -> String
|
#modulo ⇒ Object
%: a%b = a - (a.to_f/b).floor * b
740 |
# File 'sig/big_decimal.rbs', line 740
def modulo: (real | BigDecimal b) -> BigDecimal
|
#mult(other, ndigits) ⇒ Object
Returns the BigDecimal product of self and value
with a precision of ndigits decimal digits.
When ndigits is less than the number of significant digits
in the sum, the sum is rounded to that number of digits,
according to the current rounding mode; see BigDecimal.mode.
Examples:
# Set the rounding mode.
BigDecimal.mode(BigDecimal::ROUND_MODE, :half_up)
b = BigDecimal('555555.555')
b.mult(3, 0) # => 0.1666666665e7
b.mult(3, 3) # => 0.167e7
b.mult(3, 6) # => 0.166667e7
b.mult(3, 15) # => 0.1666666665e7
b.mult(3.0, 0) # => 0.1666666665e7
b.mult(Rational(3, 1), 0) # => 0.1666666665e7
b.mult(Complex(3, 0), 0) # => (0.1666666665e7+0.0i)
766 |
# File 'sig/big_decimal.rbs', line 766
def mult: (real | BigDecimal value, int digits) -> BigDecimal
|
#n_significant_digits ⇒ Integer
Returns the number of decimal significant digits in self.
BigDecimal("0").n_significant_digits # => 0
BigDecimal("1").n_significant_digits # => 1
BigDecimal("1.1").n_significant_digits # => 2
BigDecimal("3.1415").n_significant_digits # => 5
BigDecimal("-1e20").n_significant_digits # => 1
BigDecimal("1e-20").n_significant_digits # => 1
BigDecimal("Infinity").n_significant_digits # => 0
BigDecimal("-Infinity").n_significant_digits # => 0
BigDecimal("NaN").n_significant_digits # => 0
571 572 573 574 575 576 577 578 579 580 581 582 583 584 585 586 587 588 589 590 591 592 593 |
# File 'ext/bigdecimal/bigdecimal.c', line 571
static VALUE
BigDecimal_n_significant_digits(VALUE self)
{
BDVALUE v = GetBDValueMust(self);
if (VpIsZero(v.real) || !VpIsDef(v.real)) {
return INT2FIX(0);
}
ssize_t n = v.real->Prec; /* The length of frac without trailing zeros. */
for (n = v.real->Prec; n > 0 && v.real->frac[n-1] == 0; --n);
if (n == 0) return INT2FIX(0);
DECDIG x;
int nlz = BASE_FIG;
for (x = v.real->frac[0]; x > 0; x /= 10) --nlz;
int ntz = 0;
for (x = v.real->frac[n-1]; x > 0 && x % 10 == 0; x /= 10) ++ntz;
RB_GC_GUARD(v.bigdecimal);
ssize_t n_significant_digits = BASE_FIG*n - nlz - ntz;
return SSIZET2NUM(n_significant_digits);
}
|
#nan? ⇒ Boolean
Returns True if the value is Not a Number.
774 |
# File 'sig/big_decimal.rbs', line 774
def nan?: () -> bool
|
#newton_raphson_inverse(prec) ⇒ Object
3248 3249 3250 3251 |
# File 'ext/bigdecimal/bigdecimal.c', line 3248
VALUE
BigDecimal_newton_raphson_inverse(VALUE self, VALUE prec) {
return newton_raphson_inverse(self, NUM2SIZET(prec));
}
|
#nonzero? ⇒ Boolean
Returns self if the value is non-zero, nil otherwise.
782 |
# File 'sig/big_decimal.rbs', line 782
def nonzero?: () -> self?
|
#nttmult(v) ⇒ Object
3265 3266 3267 3268 3269 3270 3271 3272 3273 3274 3275 3276 3277 3278 3279 |
# File 'ext/bigdecimal/bigdecimal.c', line 3265
VALUE
BigDecimal_nttmult(VALUE self, VALUE v) {
BDVALUE a,b,c;
a = GetBDValueMust(self);
b = GetBDValueMust(v);
c = NewZeroWrap(1, VPMULT_RESULT_PREC(a.real, b.real) * BASE_FIG);
ntt_multiply(a.real->Prec, b.real->Prec, a.real->frac, b.real->frac, c.real->frac);
VpSetSign(c.real, a.real->sign * b.real->sign);
c.real->exponent = a.real->exponent + b.real->exponent;
c.real->Prec = a.real->Prec + b.real->Prec;
VpNmlz(c.real);
RB_GC_GUARD(a.bigdecimal);
RB_GC_GUARD(b.bigdecimal);
return c.bigdecimal;
}
|
#power(y, prec = 0) ⇒ BigDecimal
Returns the value raised to the power of n.
Also available as the operator **.
147 148 149 150 151 152 153 154 155 156 157 158 159 160 161 162 163 164 165 166 167 168 169 170 171 172 173 174 175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 199 200 201 202 203 204 205 206 207 208 209 210 211 212 213 214 215 216 217 218 219 220 221 222 223 224 225 226 227 228 229 230 231 232 233 234 235 236 237 238 239 240 241 242 243 244 245 246 247 248 249 250 251 252 253 254 255 256 |
# File 'lib/bigdecimal.rb', line 147 def power(y, prec = 0) prec = Internal.coerce_validate_prec(prec, :power, accept_zero: true) x = self y = Internal.coerce_to_bigdecimal(y, prec.nonzero? || n_significant_digits, :power) return Internal.nan_computation_result if x.nan? || y.nan? return BigDecimal(1) if y.zero? if y.infinite? if x < 0 return BigDecimal(0) if x < -1 && y.negative? return BigDecimal(0) if x > -1 && y.positive? raise Math::DomainError, 'Result undefined for negative base raised to infinite power' elsif x < 1 return y.positive? ? BigDecimal(0) : BigDecimal::Internal.infinity_computation_result elsif x == 1 return BigDecimal(1) else return y.positive? ? BigDecimal::Internal.infinity_computation_result : BigDecimal(0) end end if x.infinite? && y < 0 # Computation result will be +0 or -0. Avoid overflow. neg = x < 0 && y.frac.zero? && y % 2 == 1 return neg ? -BigDecimal(0) : BigDecimal(0) end if x.zero? return BigDecimal(1) if y.zero? return BigDecimal(0) if y > 0 if y.frac.zero? && y % 2 == 1 && x.sign == -1 return -BigDecimal::Internal.infinity_computation_result else return BigDecimal::Internal.infinity_computation_result end elsif x < 0 if y.frac.zero? if y % 2 == 0 return (-x).power(y, prec) else return -(-x).power(y, prec) end else raise Math::DomainError, 'Computation results in complex number' end elsif x == 1 return BigDecimal(1) end limit = BigDecimal.limit frac_part = y.frac if frac_part.zero? && prec.zero? && limit.zero? # Infinite precision calculation for `x ** int` and `x.power(int)` int_part = y.fix.to_i int_part = -int_part if (neg = int_part < 0) ans = BigDecimal(1) n = 1 xn = x while true ans *= xn if int_part.allbits?(n) n <<= 1 break if n > int_part xn *= xn # Detect overflow/underflow before consuming infinite memory if (xn.exponent.abs - 1) * int_part / n >= 0x7FFFFFFFFFFFFFFF return ((xn.exponent > 0) ^ neg ? BigDecimal::Internal.infinity_computation_result : BigDecimal(0)) * (int_part.even? || x > 0 ? 1 : -1) end end return neg ? BigDecimal(1) / ans : ans end result_prec = prec.nonzero? || [x.n_significant_digits, y.n_significant_digits, BigDecimal.double_fig].max + BigDecimal.double_fig result_prec = [result_prec, limit].min if prec.zero? && limit.nonzero? prec2 = result_prec + BigDecimal::Internal::EXTRA_PREC if y < 0 inv = x.power(-y, prec2) return BigDecimal(0) if inv.infinite? return BigDecimal::Internal.infinity_computation_result if inv.zero? return BigDecimal(1).div(inv, result_prec) end if frac_part.zero? && y.exponent < Math.log(result_prec) * 5 + 20 # Use exponentiation by squaring if y is an integer and not too large pow_prec = prec2 + y.exponent n = 1 xn = x ans = BigDecimal(1) int_part = y.fix.to_i while true ans = ans.mult(xn, pow_prec) if int_part.allbits?(n) n <<= 1 break if n > int_part xn = xn.mult(xn, pow_prec) end ans.mult(1, result_prec) else if x > 1 && x.finite? # To calculate exp(z, prec), z needs prec+max(z.exponent, 0) precision if z > 0. # Estimate (y*log(x)).exponent logx_exponent = x < 2 ? (x - 1).exponent : Math.log10(x.exponent).round ylogx_exponent = y.exponent + logx_exponent prec2 += [ylogx_exponent, 0].max end BigMath.exp(BigMath.log(x, prec2).mult(y, prec2), result_prec) end end |
#precision ⇒ Integer
Returns the number of decimal digits in self:
BigDecimal("0").precision # => 0
BigDecimal("1").precision # => 1
BigDecimal("1.1").precision # => 2
BigDecimal("3.1415").precision # => 5
BigDecimal("-1e20").precision # => 21
BigDecimal("1e-20").precision # => 20
BigDecimal("Infinity").precision # => 0
BigDecimal("-Infinity").precision # => 0
BigDecimal("NaN").precision # => 0
505 506 507 508 509 510 511 |
# File 'ext/bigdecimal/bigdecimal.c', line 505
static VALUE
BigDecimal_precision(VALUE self)
{
ssize_t precision;
BigDecimal_count_precision_and_scale(self, &precision, NULL);
return SSIZET2NUM(precision);
}
|
#precision_scale ⇒ Array
Returns a 2-length array; the first item is the result of BigDecimal#precision and the second one is of BigDecimal#scale.
See BigDecimal#precision. See BigDecimal#scale.
547 548 549 550 551 552 553 |
# File 'ext/bigdecimal/bigdecimal.c', line 547
static VALUE
BigDecimal_precision_scale(VALUE self)
{
ssize_t precision, scale;
BigDecimal_count_precision_and_scale(self, &precision, &scale);
return rb_assoc_new(SSIZET2NUM(precision), SSIZET2NUM(scale));
}
|
#quo(value) ⇒ Object #quo(value, digits) ⇒ Object
Divide by the specified value.
- digits
If specified and less than the number of significant digits of the result, the result is rounded to the given number of digits, according to the rounding mode indicated by BigDecimal.mode.
If digits is 0 or omitted, the result is the same as for the
/ operator.
See BigDecimal#/. See BigDecimal#div.
812 813 |
# File 'sig/big_decimal.rbs', line 812
def quo: (real | BigDecimal) -> BigDecimal
| (Complex) -> Complex
|
#remainder ⇒ Object
remainder
823 |
# File 'sig/big_decimal.rbs', line 823
def remainder: (real | BigDecimal) -> BigDecimal
|
#round(n, mode) ⇒ Object
Round to the nearest integer (by default), returning the result as a BigDecimal if n is specified and positive, or as an Integer if it isn't.
BigDecimal('3.14159').round #=> 3 BigDecimal('8.7').round #=> 9 BigDecimal('-9.9').round #=> -10
BigDecimal('3.14159').round(2).class.name #=> "BigDecimal" BigDecimal('3.14159').round.class.name #=> "Integer" BigDecimal('3.14159').round(0).class.name #=> "Integer"
If n is specified and positive, the fractional part of the result has no more than that many digits.
If n is specified and negative, at least that many digits to the left of the decimal point will be 0 in the result, and return value will be an Integer.
BigDecimal('3.14159').round(3) #=> 3.142 BigDecimal('13345.234').round(-2) #=> 13300
The value of the optional mode argument can be used to determine how rounding is performed; see BigDecimal.mode.
851 852 853 854 |
# File 'sig/big_decimal.rbs', line 851
def round: () -> Integer
| (int n) -> (Integer | BigDecimal)
| (int n, round_mode_integer | round_mode_symbol) -> BigDecimal
| (?int n, half: :up | :down | :even) -> BigDecimal
|
#scale ⇒ Integer
Returns the number of decimal digits following the decimal digits in self.
BigDecimal("0").scale # => 0
BigDecimal("1").scale # => 0
BigDecimal("1.1").scale # => 1
BigDecimal("3.1415").scale # => 4
BigDecimal("-1e20").scale # => 0
BigDecimal("1e-20").scale # => 20
BigDecimal("Infinity").scale # => 0
BigDecimal("-Infinity").scale # => 0
BigDecimal("NaN").scale # => 0
529 530 531 532 533 534 535 |
# File 'ext/bigdecimal/bigdecimal.c', line 529
static VALUE
BigDecimal_scale(VALUE self)
{
ssize_t scale;
BigDecimal_count_precision_and_scale(self, NULL, &scale);
return SSIZET2NUM(scale);
}
|
#sign ⇒ Object
Returns the sign of the value.
Returns a positive value if > 0, a negative value if < 0. It behaves the same with zeros - it returns a positive value for a positive zero (BigDecimal('0')) and a negative value for a negative zero (BigDecimal('-0')).
The specific value returned indicates the type and sign of the BigDecimal, as follows:
BigDecimal::SIGN_NaN:: value is Not a Number BigDecimal::SIGN_POSITIVE_ZERO:: value is +0 BigDecimal::SIGN_NEGATIVE_ZERO:: value is -0 BigDecimal::SIGN_POSITIVE_INFINITE:: value is +Infinity BigDecimal::SIGN_NEGATIVE_INFINITE:: value is -Infinity BigDecimal::SIGN_POSITIVE_FINITE:: value is positive BigDecimal::SIGN_NEGATIVE_FINITE:: value is negative
890 |
# File 'sig/big_decimal.rbs', line 890
def sign: () -> Integer
|
#split ⇒ Object
Splits a BigDecimal number into four parts, returned as an array of values.
The first value represents the sign of the BigDecimal, and is -1 or 1, or 0 if the BigDecimal is Not a Number.
The second value is a string representing the significant digits of the BigDecimal, with no leading zeros.
The third value is the base used for arithmetic (currently always 10) as an Integer.
The fourth value is an Integer exponent.
If the BigDecimal can be represented as 0.xxxxxx*10**n, then xxxxxx is the string of significant digits with no leading zeros, and n is the exponent.
From these values, you can translate a BigDecimal to a float as follows:
sign, significant_digits, base, exponent = a.split
f = sign * "0.#{significant_digits}".to_f * (base ** exponent)
(Note that the to_f method is provided as a more convenient way to translate a BigDecimal to a Float.)
920 |
# File 'sig/big_decimal.rbs', line 920
def split: () -> [ Integer, String, Integer, Integer ]
|
#sqrt(prec) ⇒ BigDecimal
Returns the square root of the value.
Result has at least prec significant digits.
262 263 264 265 266 267 268 269 270 271 272 273 274 275 276 277 278 279 280 281 282 283 |
# File 'lib/bigdecimal.rb', line 262 def sqrt(prec) prec = Internal.coerce_validate_prec(prec, :sqrt, accept_zero: true) return Internal.infinity_computation_result if infinite? == 1 raise FloatDomainError, 'sqrt of negative value' if self < 0 raise FloatDomainError, "sqrt of 'NaN'(Not a Number)" if nan? return self if zero? if prec == 0 limit = BigDecimal.limit prec = n_significant_digits + BigDecimal.double_fig prec = [limit, prec].min if limit.nonzero? end ex = exponent / 2 x = _decimal_shift(-2 * ex) y = BigDecimal(Math.sqrt(x.to_f), 0) Internal.newton_loop(prec + BigDecimal::Internal::EXTRA_PREC) do |p| y = y.add(x.div(y, p), p).div(2, p) end y._decimal_shift(ex).mult(1, prec) end |
#sub(value, digits) ⇒ Object
Subtract the specified value.
e.g. c = a.sub(b,n)
- digits
If specified and less than the number of significant digits of the result, the result is rounded to that number of digits, according to BigDecimal.mode.
946 |
# File 'sig/big_decimal.rbs', line 946
def sub: (real | BigDecimal value, int digits) -> BigDecimal
|
#to_d ⇒ BigDecimal
Returns self.
require 'bigdecimal/util'
d = BigDecimal("3.14")
d.to_d # => 0.314e1
110 111 112 |
# File 'lib/bigdecimal/util.rb', line 110 def to_d self end |
#to_digits ⇒ String
Converts a BigDecimal to a String of the form "nnnnnn.mmm". This method is deprecated; use BigDecimal#to_s("F") instead.
require 'bigdecimal/util'
d = BigDecimal("3.14")
d.to_digits # => "3.14"
90 91 92 93 94 95 96 97 98 |
# File 'lib/bigdecimal/util.rb', line 90 def to_digits if self.nan? || self.infinite? || self.zero? self.to_s else i = self.to_i.to_s _,f,_,z = self.frac.split i + "." + ("0"*(-z)) + f end end |
#to_f ⇒ Object
Returns a new Float object having approximately the same value as the BigDecimal number. Normal accuracy limits and built-in errors of binary Float arithmetic apply.
956 |
# File 'sig/big_decimal.rbs', line 956
def to_f: () -> Float
|
#to_i ⇒ Object
Returns the value as an Integer.
If the BigDecimal is infinity or NaN, raises FloatDomainError.
966 |
# File 'sig/big_decimal.rbs', line 966
def to_i: () -> Integer
|
#to_int ⇒ Object
Returns the value as an Integer.
If the BigDecimal is infinity or NaN, raises FloatDomainError.
973 |
# File 'sig/big_decimal.rbs', line 973
def to_int: () -> Integer
|
#to_r ⇒ Object
Converts a BigDecimal to a Rational.
981 |
# File 'sig/big_decimal.rbs', line 981
def to_r: () -> Rational
|
#to_s(s) ⇒ Object
Converts the value to a string.
The default format looks like 0.xxxxEnn.
The optional parameter s consists of either an integer; or an optional '+' or ' ', followed by an optional number, followed by an optional 'E' or 'F'.
If there is a '+' at the start of s, positive values are returned with a leading '+'.
A space at the start of s returns positive values with a leading space.
If s contains a number, a space is inserted after each group of that many digits, starting from '.' and counting outwards.
If s ends with an 'E', scientific notation (0.xxxxEnn) is used.
If s ends with an 'F', conventional floating point notation is used.
Examples:
BigDecimal('-1234567890123.45678901234567890').to_s('5F')
#=> '-123 45678 90123.45678 90123 45678 9'
BigDecimal('1234567890123.45678901234567890').to_s('+8F')
#=> '+12345 67890123.45678901 23456789'
BigDecimal('1234567890123.45678901234567890').to_s(' F')
#=> ' 1234567890123.4567890123456789'
1017 |
# File 'sig/big_decimal.rbs', line 1017
def to_s: (?String | int s) -> String
|
#truncate(n) ⇒ Object
Truncate to the nearest integer (by default), returning the result as a BigDecimal.
BigDecimal('3.14159').truncate #=> 3 BigDecimal('8.7').truncate #=> 8 BigDecimal('-9.9').truncate #=> -9
If n is specified and positive, the fractional part of the result has no more than that many digits.
If n is specified and negative, at least that many digits to the left of the decimal point will be 0 in the result.
BigDecimal('3.14159').truncate(3) #=> 3.141 BigDecimal('13345.234').truncate(-2) #=> 13300.0
1039 1040 |
# File 'sig/big_decimal.rbs', line 1039
def truncate: () -> Integer
| (int n) -> BigDecimal
|
#vpdivd(r, cprec) ⇒ Object
3238 3239 3240 3241 |
# File 'ext/bigdecimal/bigdecimal.c', line 3238
VALUE
BigDecimal_vpdivd(VALUE self, VALUE r, VALUE cprec) {
return BigDecimal_vpdivd_generic(self, r, cprec, VpDivdNormal);
}
|
#vpdivd_newton(r, cprec) ⇒ Object
3243 3244 3245 3246 |
# File 'ext/bigdecimal/bigdecimal.c', line 3243
VALUE
BigDecimal_vpdivd_newton(VALUE self, VALUE r, VALUE cprec) {
return BigDecimal_vpdivd_generic(self, r, cprec, VpDivdNewton);
}
|
#vpmult(v) ⇒ Object
3253 3254 3255 3256 3257 3258 3259 3260 3261 3262 3263 |
# File 'ext/bigdecimal/bigdecimal.c', line 3253
VALUE
BigDecimal_vpmult(VALUE self, VALUE v) {
BDVALUE a,b,c;
a = GetBDValueMust(self);
b = GetBDValueMust(v);
c = NewZeroWrap(1, VPMULT_RESULT_PREC(a.real, b.real) * BASE_FIG);
VpMult(c.real, a.real, b.real);
RB_GC_GUARD(a.bigdecimal);
RB_GC_GUARD(b.bigdecimal);
return c.bigdecimal;
}
|
#zero? ⇒ Boolean
Returns True if the value is zero.
1048 |
# File 'sig/big_decimal.rbs', line 1048
def zero?: () -> bool
|